@@ -2,7 +2,7 @@ import uuid as uuid_module
 from enum import IntEnum
 from typing import Iterable, List, Union
 
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from rest_framework.serializers import Serializer
 
 from .settings import DYNAMICFORMS
@@ -10,15 +10,19 @@ from .settings import DYNAMICFORMS
 
 class ActionBase(object):
 
-    def __init__(self, action_js: str, name: Union[str, None] = None, serializer: Serializer = None):
+    def __init__(self, name: str, serializer: Serializer = None, action=None):
         """
-        :param action_js: JavaScript to execute when action is run
         :param name: name by which to recognise this action in further processing, e.g. Serializer.suppress_action
         :param serializer: bind to this serializer instance
+        :param action: this is a dict specifying how the vue renderer should render the action. several modes are
+           supported:
+           dict(func_name, params): calls window.func_name(params) (func_name can contain dots which will be resolved)
+           dict(href): action is actually a href - the browser will redirect to the given href
+           dict(href, router_name=True): action is a href to a Vue router named path. href will be resolved from name
         """
+        assert name is not None, 'Action name must not be None'
         self.name = name
-        self.action_js = action_js
-        assert self.action_js is not None, 'When declaring action, it must declare action JavaScript to execute'
+        self.action = action
         # serializer will be set when obtaining a resolved copy
         self.serializer = serializer
 
@@ -51,25 +55,8 @@ class ActionBase(object):
             string = string.replace(key, val)
         return string
 
-
-class RenderableActionMixin(object):
-    """
-    Action that is rendered on screen
-    """
-
-    def __init__(self, label: str, title: str, icon: str = None, btn_classes: Union[str, dict, None] = None):
-        """
-        :param label: Label for rendering to on screen control
-        :param title: Hint text for on-screen control
-        :param icon: optional icon to add to render
-        :param btn_classes: optional class(es) of button. if variable is dict and key 'replace' have value True,
-         then default class will be replaced with class(es) that are under key 'classes'. In other case class(es) will
-         be just added to default class
-        """
-        self.label = label
-        self.title = title
-        self.icon = icon
-        self.btn_classes = btn_classes
+    def as_component_def(self):
+        return {k: getattr(self, k) for k in ('name', 'action')}
 
 
 class TablePosition(IntEnum):
@@ -86,31 +73,100 @@ class TablePosition(IntEnum):
     FIELD_END = 6  # On right side of field value
 
 
+class FormButtonTypes(IntEnum):
+    CANCEL = 1
+    SUBMIT = 2
+    CUSTOM = 3
+
+
+class RenderableActionMixin(object):
+    """
+    Action that is rendered on screen
+    """
+
+    def __init__(self, label: str, title: str, icon: str = None, btn_classes: Union[str, dict, None] = None,
+                 display_style: Union[dict, None] = None):
+        """
+        :param label: Label for rendering to on screen control
+        :param title: Hint text for on-screen control
+        :param icon: optional icon to add to render
+        :param btn_classes: optional class(es) of button. if variable is dict and key 'replace' have value True,
+         then default class will be replaced with class(es) that are under key 'classes'. In other case class(es) will
+         be just added to default class
+        :param display_style: How is action rendered - Button or non button, only icon, only label, bot icon and label
+        """
+        self.label = label
+        self.title = title
+        self.icon = icon
+        self.btn_classes = btn_classes
+        self.display_style = display_style
+
+    def as_component_def(self):
+        return dict(label=self.label, title=self.title, icon=self.icon, classes=self.btn_classes,
+                    displayStyle=self.display_style)
+
+    @staticmethod
+    def def_display_style(table_position: Union[TablePosition, None] = None,
+                          form_button: Union[FormButtonTypes, None] = None):
+        if table_position == TablePosition.HEADER:
+            return dict(md=dict(asButton=True, showLabel=True, showIcon=True),
+                        xs=dict(asButton=True, showLabel=False, showIcon=True))
+        elif table_position in (
+            TablePosition.FILTER_ROW_START, TablePosition.FILTER_ROW_END, TablePosition.ROW_START, TablePosition.ROW_END
+        ):
+            return dict(xs=dict(asButton=True, showLabel=False, showIcon=True))
+        elif table_position in (TablePosition.FIELD_START, TablePosition.FIELD_END):
+            return dict(xs=dict(asButton=False, showLabel=False, showIcon=True))
+        elif form_button in (FormButtonTypes.SUBMIT, FormButtonTypes.CANCEL, FormButtonTypes.CUSTOM):
+            return dict(xs=dict(asButton=True, showLabel=True, showIcon=False))
+        return None
+
+
 class TableAction(ActionBase, RenderableActionMixin):
 
-    def __init__(self, position: TablePosition, label: str, action_js: str,
-                 title: Union[str, None] = None, icon: Union[str, None] = None, field_name: Union[str, None] = None,
+    def __init__(self, position: TablePosition, label: str, title: Union[str, None] = None,
+                 icon: Union[str, None] = None, field_name: Union[str, None] = None,
                  name: Union[str, None] = None, serializer: Serializer = None,
-                 btn_classes: Union[str, dict, None] = None):
-        ActionBase.__init__(self, action_js, name, serializer)
-        RenderableActionMixin.__init__(self, label, title, icon, btn_classes)
+                 btn_classes: Union[str, dict, None] = None, action=None, display_style=None):
+        ActionBase.__init__(self, name, serializer, action=action)
+        RenderableActionMixin.__init__(self, label, title, icon, btn_classes,
+                                       display_style or self.def_display_style(table_position=position))
         self.position = position
         self.field_name = field_name
 
     def copy_and_resolve_reference(self, serializer: Serializer):
-        return TableAction(self.position, self.label, self.action_js, self.title, self.icon, self.field_name, self.name,
-                           serializer, self.btn_classes)
+        return TableAction(self.position, self.label, self.title, self.icon, self.field_name, self.name,
+                           serializer, self.btn_classes, action=self.action, display_style=self.display_style)
+
+    def to_component_params(self, row_data, serializer):
+        """
+        generates a dict with parameters for component that is going to represent this action.
+        none means don't render / activate this action on this row
+
+        :param row_data:
+        :param serializer:
+        :return:
+        """
+        if self.position in (TablePosition.HEADER, TablePosition.FILTER_ROW_START, TablePosition.FILTER_ROW_END):
+            return self.name, None
+        return self.name, {}
+
+    def as_component_def(self):
+        res = dict(position=self.position.name, field_name=self.field_name)
+        res.update(ActionBase.as_component_def(self))
+        res.update(RenderableActionMixin.as_component_def(self))
+        return res
 
     def render(self, serializer: Serializer, **kwds):
         ret = rowclick = rowrclick = ''
         stop_propagation = 'dynamicforms.stopEventPropagation(event);'
 
-        action_action = self.prepare_string(self.action_js)
+        action_action = self.prepare_string('')  # TODO maybe: self.action_js. more likely this just needs to be removed
         if self.position != TablePosition.HEADER:
             # We need to do this differently because of dynamic page loading for tables: each time the serializer
             # has a different UUID
             action_action = action_action.replace('__TABLEID__',
-                                                  "$(event.target).parents('table').attr('id').substr(5)")
+                                                  "$(event.target).parents('table').attr('id').substring(5)")
         else:
             action_action = action_action.replace('__TABLEID__', "'" + str(serializer.uuid) + "'")
 
@@ -119,8 +175,6 @@ class TableAction(ActionBase, RenderableActionMixin):
         elif self.position == TablePosition.ROW_RIGHTCLICK:
             rowrclick = action_action
         else:
-            from uuid import uuid1
-
             def get_btn_class(default_class, additional_class):
                 classes = default_class.split(' ')
                 if additional_class:
@@ -136,7 +190,7 @@ class TableAction(ActionBase, RenderableActionMixin):
             button_name = (' name="btn-%s" ' % self.name) if self.name else ''
             btn_class = get_btn_class("btn btn-info", self.btn_classes)
 
-            btnid = uuid1()
+            btnid = uuid_module.uuid1()
             ret += '<button id="df-action-btn-{btnid}" type="button" class="{btn_class}" title="{title}"{button_name}' \
                    'onClick="{stop_propagation} {action}">{icon_def}{label}</button>'. \
                 format(btnid=btnid, stop_propagation=stop_propagation, action=action_action,
@@ -170,20 +224,38 @@ class TableAction(ActionBase, RenderableActionMixin):
 
 class FieldChangeAction(ActionBase):
 
-    def __init__(self, tracked_fields: Iterable[str], action_js: str, name: Union[str, None] = None,
-                 serializer: Serializer = None):
-        super().__init__(action_js, name, serializer)
-        self.tracked_fields = tracked_fields
-        assert self.tracked_fields, 'When declaring an action, it must track at least one form field'
+    def __init__(self, tracked_fields: Iterable[str], name: Union[str, None] = None,
+                 serializer: Serializer = None, action=None):
+        super().__init__(name, serializer, action=action)
+        self.tracked_fields = tracked_fields or []
+        # assert self.tracked_fields, 'When declaring an action, it must track at least one form field'
         if serializer:
             self.tracked_fields = [self._resolve_reference(f) for f in self.tracked_fields]
 
+    def to_component_params(self, row_data, serializer):
+        """
+        generates a dict with parameters for component that is going to represent this action.
+        none means don't render / activate this action on this row
+
+        :param row_data:
+        :param serializer:
+        :return:
+        """
+        # perhaps this function is misnamed: it should be row_render_params, because only the row-level properties are
+        #  processed
+        return self.name, None
+
+    def as_component_def(self):
+        res = dict(position='VALUE_CHANGED', tracked_fields=list(self.tracked_fields))
+        res.update(ActionBase.as_component_def(self))
+        return res
+
     def _resolve_reference(self, ref):
-        from .mixins import RenderMixin
+        from .mixins import FieldRenderMixin
 
         if isinstance(ref, uuid_module.UUID):
             return str(ref)
-        elif isinstance(ref, RenderMixin):
+        elif isinstance(ref, FieldRenderMixin):
             # TODO unit tests!!!
             # TODO test what happens if the Field instance given is from another serializer
             # TODO test what happens when Field instance is actually a Serializer (when should onchange trigger for it?)
@@ -199,7 +271,7 @@ class FieldChangeAction(ActionBase):
         raise Exception('Unknown reference type for Action tracked field (%r)' % ref)
 
     def copy_and_resolve_reference(self, serializer: Serializer):
-        return FieldChangeAction(self.tracked_fields, self.action_js, self.name, serializer)
+        return FieldChangeAction(self.tracked_fields, self.name, serializer)
 
     def render(self, serializer: Serializer, **kwds):
         res = 'var action_func{0.action_id} = {0.action_js};\n'.format(self)
@@ -212,7 +284,20 @@ class FieldChangeAction(ActionBase):
 class FormInitAction(ActionBase):
 
     def copy_and_resolve_reference(self, serializer: Serializer):
-        return FormInitAction(self.action_js, self.name, serializer)
+        return FormInitAction(self.name, serializer, action=self.action)
+
+    def to_component_params(self, row_data, serializer):
+        """
+        generates a dict with parameters for component that is going to represent this action.
+        none means don't render / activate this action on this row
+
+        :param row_data:
+        :param serializer:
+        :return:
+        """
+        # perhaps this function is misnamed: it should be row_render_params, because only the row-level properties are
+        #  processed
+        return self.name, None
 
     def render(self, serializer: Serializer, **kwds):
         # we need window.setTimeout because at the time of form generation, the initial fields value collection
@@ -222,8 +307,21 @@ class FormInitAction(ActionBase):
 
 class FieldInitAction(FieldChangeAction):
 
+    def to_component_params(self, row_data, serializer):
+        """
+        generates a dict with parameters for component that is going to represent this action.
+        none means don't render / activate this action on this row
+
+        :param row_data:
+        :param serializer:
+        :return:
+        """
+        # perhaps this function is misnamed: it should be row_render_params, because only the row-level properties are
+        #  processed
+        return self.name, None
+
     def copy_and_resolve_reference(self, serializer: Serializer):
-        return FieldInitAction(self.tracked_fields, self.action_js, self.name, serializer)
+        return FieldInitAction(self.tracked_fields, self.name, serializer)
 
     def render(self, serializer: Serializer, **kwds):
         # we need window.setTimeout because at the time of form generation, the initial fields value collection
@@ -231,52 +329,78 @@ class FieldInitAction(FieldChangeAction):
         return 'window.setTimeout(function() {{ {0.action_js} }}, 1);;\n'.format(self)
 
 
-class FormButtonTypes(IntEnum):
-    CANCEL = 1
-    SUBMIT = 2
-    CUSTOM = 3
+class FormPosition(IntEnum):
+    FORM_HEADER = 1
+    FORM_FOOTER = 2
 
 
-class FormButtonAction(ActionBase):
+class FormButtonAction(ActionBase, RenderableActionMixin):
     DEFAULT_LABELS = {
         FormButtonTypes.CANCEL: _('Cancel'),
         FormButtonTypes.SUBMIT: _('Save changes'),
         FormButtonTypes.CUSTOM: _('Custom'),
     }
 
-    def __init__(self, btn_type: FormButtonTypes, label: str = None, btn_classes: str = None, action_js: str = None,
-                 button_is_primary: bool = None, positions: List[str] = None,
-                 name: Union[str, None] = None, serializer: Serializer = None):
-        super().__init__(action_js or False, name, serializer)
+    def __init__(self, btn_type: FormButtonTypes, label: str = None, btn_classes: str = None,
+                 button_is_primary: bool = None, position: FormPosition = FormPosition.FORM_FOOTER,
+                 name: Union[str, None] = None, serializer: Serializer = None, icon: Union[str, None] = None,
+                 action=None, display_style=None):
+        ActionBase.__init__(self, name, serializer, action=action)
+        title = label
+        label = label or FormButtonAction.DEFAULT_LABELS[btn_type or FormButtonTypes.CUSTOM]
+        RenderableActionMixin.__init__(self, label, title, icon, btn_classes,
+                                       display_style or self.def_display_style(form_button=btn_type))
         self.uuid = uuid_module.uuid1()
         self.btn_type = btn_type
-        self.label = label or FormButtonAction.DEFAULT_LABELS[btn_type or FormButtonTypes.CUSTOM]
-        self.positions = positions or ['dialog', 'form']
+        self.position = position or FormPosition.FORM_FOOTER
 
         if button_is_primary is None:
             button_is_primary = btn_type == FormButtonTypes.SUBMIT
         self.button_is_primary = button_is_primary
 
+        DF = DYNAMICFORMS
         self.btn_classes = btn_classes or (
-            DYNAMICFORMS.form_button_classes + ' '
-            + (DYNAMICFORMS.form_button_classes_primary if button_is_primary
-               else DYNAMICFORMS.form_button_classes_secondary) + ' '
-            + (DYNAMICFORMS.form_button_classes_cancel if btn_type == FormButtonTypes.CANCEL else '')
+            DF.form_button_classes + ' ' +
+            (DF.form_button_classes_primary if button_is_primary else DF.form_button_classes_secondary) + ' ' +
+            (DF.form_button_classes_cancel if btn_type == FormButtonTypes.CANCEL else '')
         )
 
-    def copy_and_resolve_reference(self, serializer):
-        return FormButtonAction(self.btn_type, self.label, self.btn_classes, self.action_js, self.button_is_primary,
-                                self.positions, self.name, serializer)
+    def to_component_params(self, row_data, serializer):
+        """
+        generates a dict with parameters for component that is going to represent this action.
+        none means don't render / activate this action on this row
 
-    def render(self, serializer: Serializer, position=None, **kwds):
+        :param row_data:
+        :param serializer:
+        :return:
+        """
+        # perhaps this function is misnamed: it should be row_render_params, because only the row-level properties are
+        #  processed
+        return self.name, None
+
+    def as_component_def(self):
+        res = dict(uuid=str(self.uuid), element_id=f'{self.name}-{self.serializer.uuid}', type=self.btn_type.name,
+                   position=self.position.name)
+        res.update(ActionBase.as_component_def(self))
+        res.update(RenderableActionMixin.as_component_def(self))
+        return res
+
+    def copy_and_resolve_reference(self, serializer):
+        return FormButtonAction(self.btn_type, self.label, self.btn_classes, self.button_is_primary,
+                                self.position, self.name, serializer)
+
+    def render(self, serializer: Serializer, position: FormPosition = None, **kwds):
         if self.btn_type == FormButtonTypes.CANCEL and position == 'form':
             return ''
-        action_js = self.action_js
+        action_js = None  # TODO self.action_js
         button_name = ('name="btn-%s"' % self.name) if self.name else ''
         if isinstance(action_js, str):
             action_js = self.prepare_string(action_js)
             action_js = action_js.format(**locals())
-        button_type = 'button' if self.btn_type != FormButtonTypes.SUBMIT or position == 'dialog' else 'submit'
+
+        is_submit = self.btn_type != FormButtonTypes.SUBMIT or position == FormPosition.FORM_FOOTER
+        button_type = 'submit' if is_submit else 'button'
+
         data_dismiss = 'data-dismiss="modal"' if self.btn_type == FormButtonTypes.CANCEL else ''
         if self.btn_type == FormButtonTypes.SUBMIT:
             button_id = 'save-' + str(serializer.uuid)
@@ -305,31 +429,33 @@ class Actions(object):
             self.actions = []
             return
 
-        self.actions = list(args)  # type: List[ActionBase]
+        self.actions: List[ActionBase] = list(args)
         if add_default_crud:
             self.actions.append(
-                TableAction(TablePosition.HEADER, _('+ Add'), title=_('Add new record'), name='add',
-                            action_js="dynamicforms.newRow('{% url url_reverse|add:'-detail' pk='new' format='html' %}'"
-                                      ", 'record', __TABLEID__);")
+                TableAction(TablePosition.HEADER, _('Add'), title=_('Add new record'), name='add',
+                            icon='add-circle-outline')
             )
             self.actions.append(
                 TableAction(TablePosition.ROW_CLICK, _('Edit'), title=_('Edit record'), name='edit',
-                            action_js="dynamicforms.editRow('{% url url_reverse|add:'-detail' pk='__ROWID__' "
-                                      "format='html' %}'.replace('__ROWID__', $(event.target.parentElement).closest("
-                                      "'tr[class=\"df-table-row\"]').attr('data-id')), 'record', __TABLEID__);")
+                            icon='pencil-outline')
             )
             self.actions.append(
                 TableAction(TablePosition.ROW_END, label=_('Delete'), title=_('Delete record'), name='delete',
-                            action_js="dynamicforms.deleteRow('{% url url_reverse|add:'-detail' pk=row.id %}', "
-                                      "{{row.id}}, 'record', __TABLEID__);"))
+                            icon='trash-outline')
+            )
+            self.actions.append(
+                TableAction(TablePosition.ROW_CLICK, _('Sort'), title=_('Sort by column'), name='sort')
+            )
         if add_default_filter:
             self.actions.append(TableAction(TablePosition.HEADER, label=_('Filter'), title=_('Filter'), name='filter',
-                                            action_js="dynamicforms.defaultFilter(event);"))
+                                            icon='search-outline'))
 
         if add_form_buttons:
             self.actions.append(FormButtonAction(btn_type=FormButtonTypes.CANCEL, name='cancel'))
             self.actions.append(FormButtonAction(btn_type=FormButtonTypes.SUBMIT, name='submit'))
 
+        self.actions.append(FieldChangeAction(None, 'value_changed'))
+
     def get_resolved_copy(self, serializer) -> 'Actions':
         """
         Returns a copy of declared actions bound to the serializer
@@ -421,8 +547,10 @@ class Actions(object):
         res = ''
 
         for button in self.actions:
-            if isinstance(button, FormButtonAction) and position in button.positions and \
-                    not serializer.suppress_action(button, request, viewset):
+            if (
+                isinstance(button, FormButtonAction) and position == button.position and
+                not serializer.suppress_action(button, request, viewset)
+            ):
                 res += button.render(serializer, position=position)
         return res
 
