@@ -1,15 +1,24 @@
 # flake8: noqa
+import abc
+
+from rest_framework.fields import Field as DRFField
 
 from .action import ActionMixin
-from .allow_tags import AllowTagsMixin
+from .choice import ChoiceMixin
 from .enable_copy import EnableCopyMixin
+from .field_render import DisplayMode, FieldAlignment, FieldRenderMixin
 from .file_field import FileFieldMixin
 from .help_text import FieldHelpTextMixin
 from .implicit_hidden import HiddenFieldMixin
 from .implicit_natural_date import DateFieldMixin, DateTimeFieldMixin, TimeFieldMixin
-from .null_choice import NullChoiceMixin
 from .null_value import NullValueMixin
 from .password_field import PasswordFieldMixin
 from .related_field_ajax import RelatedFieldAJAXMixin
-from .render import DisplayMode, RenderMixin
 from .rtf_field import RTFFieldMixin
+
+
+class DFField(FieldRenderMixin, ActionMixin, FieldHelpTextMixin, DRFField, abc.ABC):
+    """
+    Class only for type hinting
+    """
+    pass
